@@ -68,8 +68,8 @@ word32 F(blf_ctx * bc, word32 x)
 
 /*
   y=bc->S[0][a] + bc->S[1][b];
-  y = y ^ bc->S[2][c];
-  y = y + bc->S[3][d];
+  y ^= bc->S[2][c];
+  y += bc->S[3][d];
 */
   y = ((bc->S[0][a] + bc->S[1][b]) ^ bc->S[2][c]) + bc->S[3][d];
      
@@ -92,8 +92,8 @@ void enblf(blf_ctx *c,word32 *x)
 #endif 
   
  for (i=0; i<N;++i) {
-  Xl = Xl ^ c->P[i];
-  Xr = F(c,Xl) ^ Xr;
+  Xl ^= c->P[i];
+  Xr ^= F(c,Xl);
   
   temp=Xl;
   Xl=Xr;
@@ -104,8 +104,8 @@ void enblf(blf_ctx *c,word32 *x)
  Xl=Xr;
  Xr=temp;
  
- Xr = Xr ^ c->P[N];
- Xl = Xl ^ c->P[N+1];
+ Xr ^= c->P[N];
+ Xl ^= c->P[N+1];
  
 #ifdef WORDS_BIGENDIAN
  x[0] = Xl;
@@ -128,8 +128,8 @@ void enblf_noswap(blf_ctx *c,word32 *x) /* Used internally */
  Xr=x[1];
  
  for (i=0; i<N;++i) {
-  Xl = Xl ^ c->P[i];
-  Xr = F(c,Xl) ^ Xr;
+  Xl ^= c->P[i];
+  Xr ^= F(c,Xl);
   
   temp=Xl;
   Xl=Xr;
@@ -140,8 +140,8 @@ void enblf_noswap(blf_ctx *c,word32 *x) /* Used internally */
  Xl=Xr;
  Xr=temp;
  
- Xr = Xr ^ c->P[N];
- Xl = Xl ^ c->P[N+1];
+ Xr ^= c->P[N];
+ Xl ^= c->P[N+1];
  
  x[0] = Xl;
  x[1] = Xr;
@@ -161,8 +161,8 @@ void deblf(blf_ctx *c,word32 *x)
 #endif  
   
   for (i=N+1;i>1;--i) { 
-    Xl = Xl ^ c->P[i];
-    Xr = F(c,Xl) ^ Xr;
+    Xl ^= c->P[i];
+    Xr ^= F(c,Xl);
     
     temp=Xl;
     Xl=Xr;
@@ -173,8 +173,8 @@ void deblf(blf_ctx *c,word32 *x)
   Xl=Xr;
   Xr=temp;
   
-  Xr = Xr ^ c->P[1];
-  Xl = Xl ^ c->P[0];  
+  Xr ^= c->P[1];
+  Xl ^= c->P[0];  
 
 #ifdef WORDS_BIGENDIAN
   x[0]=Xl;
