@@ -53,7 +53,7 @@ static keyInstance *KI=NULL;    /* For Twofish */
 static char **kn1=NULL; /* For DES */
 static char **kn2=NULL;
 static char **kn3=NULL; /* For tripleDES */
-static unsigned char** saferkey=NULL; /* For safer64 and 128 */
+static word8** saferkey=NULL; /* For safer64 and 128 */
 static int finished_threads=0;
 
 /* For CBC mode */
@@ -62,13 +62,13 @@ static word32 **previous_plaintext;
 static word32 **previous_cipher;
 
 /* For CFB mode */
-static char **sd_register;
-static char **s_register;
-static char **enc_s_register;
-static char **enc_sd_register;
+static word8 **sd_register;
+static word8 **s_register;
+static word8 **enc_s_register;
+static word8 **enc_sd_register;
 
 /* hold the key and the algorithm */
-static unsigned char **keyword_given=NULL;
+static word8 **keyword_given=NULL;
 static int *algorithm_given=NULL;
 
 int get_block_size(int algorithm)
