@@ -67,30 +67,40 @@
 
 #define CRYPT 5
 
-#if SIZEOF_LONG_INT == 4
+#if SIZEOF_UNSIGNED_LONG_INT == 4
  typedef unsigned long word32;
-#elif SIZEOF_INT == 4
+#elif SIZEOF_UNSIGNED_INT == 4
  typedef unsigned int word32;
 #else
 # error "Cannot find a 32 bit integer in your system, sorry."
 #endif
 
-#if SIZEOF_INT == 2
+#if SIZEOF_UNSIGNED_INT == 2
  typedef unsigned int word16;
-#elif SIZEOF_SHORT_INT == 2
+#elif SIZEOF_UNSIGNED_SHORT_INT == 2
  typedef unsigned short word16;
 #else 
 # error "Cannot find a 16 bit integer in your system, sorry."
 #endif
 
-#if SIZEOF_CHAR == 1
+#if SIZEOF_UNSIGNED_CHAR == 1
  typedef unsigned char word8; 
 #else
 # error "Cannot find an 8 bit char in your system, sorry."
 #endif
 
-/* originally was in 3way.c */
-#define RAND32 (word32) ((word32)rand() << 17 ^ (word32)rand() << 9 ^ rand())
+/* originally was defined in 3way.c */
+#ifndef HAVE_DEV_RANDOM
+# define RAND32 (word32) ((word32)rand() << 17 ^ (word32)rand() << 9 ^ rand())
+# define SRAND32 RAND32
+# define RAND8 (word8) ((word8)rand() << 4 ^ (word8)rand() << 2 ^ rand())
+# define SRAND8 RAND8
+#else
+# define RAND32 get_rand32()
+# define RAND8 get_rand8()
+# define SRAND32 get_safe_rand32()
+# define SRAND8 get_safe_rand8()
+#endif
 
 #ifndef HAVE_MEMMOVE
 # ifdef HAVE_BCOPY
