@@ -46,7 +46,7 @@ char *btmp;
    btmp=getpass("Re-Enter key: ");
    
   if (strcmp(ztmp,btmp)!=0) {
-     fprintf(stderr,_("Keywords do not match.\n"));
+     fprintf(stderr,_("Keywords do not match or they are too long.\n"));
      return NULL;
   }
 
@@ -68,6 +68,7 @@ char msg[100];
 
    if ((strlen(tmp) < numofchar) && (type!=BLOWFISH)) { fprintf(stderr,_("Warning: keyword is too small for this algorithm\n")); }
    if (strlen(tmp) > numofchar) { fprintf(stderr,_("Warning: Reducing the size of keyword\n")); }
+   *len=strlen(tmp);
  }
  else { /* 7bit mode */
 
@@ -284,18 +285,19 @@ if (stream_flag==FALSE) remove(outfile); /* Delete the file we were writing to *
 char** read_key_file(char * file, int* num) {
 
   FILE* FROMF;
-  char keyword[127], **keys=NULL;
+  char keyword[128], **keys=NULL;
   int x=0;
 
   FROMF=fopen(file,"r");
   if (FROMF==NULL) {perror("fopen");exit(1);}
   if (read_lock( fileno((FILE*)FROMF))==-1) return 0;
 
-  while (fgets( keyword, 127, FROMF)!=NULL) {
+  while (fgets( keyword, 128, FROMF)!=NULL) {
     x++;
-
     keys=realloc(keys, x*sizeof(char*));
     keys[x-1]=malloc(strlen(keyword)+1);
+/* Remove newline */
+    if (keyword[strlen(keyword)-1]=='\n') keyword[strlen(keyword)-1]='\0';
     strcpy( keys[x-1], keyword);
 
   }
