@@ -17,3 +17,6 @@ void copyDate ( char *srcName, char *dstName );
 int check_file(char * filename);
 int is_normal_file(char *filename);
 #endif
+
+unsigned int Hex_To_Int(char ch);
+void Read_Key_Hex_String(char *str);
