@@ -33,21 +33,24 @@
 #include <crc32.h>
 #include <locks.h>
 #include <enigma.h>
+#include <hex.h>
+#include <keys.h>
+#include <random.h>
 
- unsigned int bit7_flag=BITS7;
+ unsigned int bit7_flag=-1, hexflag=FALSE;
  unsigned int stream_flag=FALSE;
  unsigned int mode=CBC; /* 0:cbc 1:ecb */
  unsigned int type=BLOWFISH; /* default: blowfish */
  int quiet=TRUE; /* silent by default */
  int unlink_flag=FALSE;
- int bare_flag=FALSE;
+ int bare_flag=FALSE, real_random_flag=FALSE;
  char *outfile=0;
 
 
 void usage(char *name)
 {
 
-     fprintf(stderr,_("Usage: mcrypt [-dLFu87bhv] [-f keyfile] [-k keyword] [-m mode] [-a algorithm] [file ...]\n\n"));
+     fprintf(stderr,_("Usage: mcrypt [-dLFu87bhvrx] [-f keyfile] [-k keyword] [-m mode] [-a algorithm] [file ...]\n\n"));
 
      fprintf(stderr,_("mini-crypt encrypts/decrypts files using several symmetric block algorithms.\n"));
      fprintf(stderr,_("Legal parameters are:\n\
@@ -60,11 +63,13 @@ void usage(char *name)
  -m --mode=MODE               specify the mode (cfb, ecb or cbc).\n\
                               (Default is cbc)\n\
  -k --key=KEYWORD             keyword to use.\n\
- -f --file=FILENAME           file to read the keyword from.\n\
+ -f --keyfile=FILENAME        file to read the keyword from.\n\
+ -x --hex                     keyword in hex.\n\
  -7 --7bit                    keyword in 7bit mode. (default)\n\
  -8 --8bit                    keyword in 8bit mode.\n\
  -u --unlink                  unlink (and destroy) the input file after\n\
                               encryption/decryption\n\
+ -r --random                  Use real random data (if your system supports it).\n\
  -L --license                 displays license information.\n\
  -q --quiet                   suppress some non critical warnings.\n\
  -V --verbose                 some more information is displayed.\n\
@@ -99,7 +104,9 @@ struct option longopts[] = {
 	    { "mode", 1, 0, 'm' },
 	    { "algorithm", 1, 0, 'a' },
 	    { "key", 1, 0, 'k' },
-	    { "file", 1, 0, 'f' },
+	    { "hex", 0, 0, 'x' },
+	    { "random", 0, 0, 'r' },	    
+	    { "keyfile", 1, 0, 'f' },
 	    { "force", 0, 0, 'F' },
 	    { "7bit", 0, 0, '7' },	    
 	    { "8bit", 0, 0, '8' },
@@ -111,12 +118,12 @@ struct option longopts[] = {
        	  };
 
 #ifdef HAVE_SIGNAL_H
- Signal( SIGINT, shandler);
+ Signal( SIGINT , shandler);
  Signal( SIGQUIT, shandler);
  Signal( SIGSEGV, shandler);
  Signal( SIGPIPE, shandler);   
  Signal( SIGTERM, shandler);   
- Signal( SIGHUP, shandler);   
+ Signal( SIGHUP , shandler);   
  Signal( SIGUSR1, SIG_IGN);
  Signal( SIGUSR2, SIG_IGN);
  Signal( SIGALRM, SIG_IGN);
@@ -129,7 +136,9 @@ struct option longopts[] = {
 #endif
 
 /* For RAND32... Called only here */
+#ifndef HAVE_DEV_RANDOM
  srand(time(0));
+#endif
 
 /* Examine how we were called */
    if (strstr(argv[0],"decrypt")!=NULL) {
@@ -148,7 +157,7 @@ struct option longopts[] = {
 
         file=malloc((argc-optind)*sizeof(char*)); 
 
-	while( (ch=getopt_long(argc,argv,"F78bLVuqda:m:k:f:hv",longopts, (int *)0 )) != EOF )
+	while( (ch=getopt_long(argc,argv,"F78bLVxuqrda:m:k:f:hv",longopts, (int *)0 )) != EOF )
 
 		switch(ch) {
 	
@@ -182,6 +191,15 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 			ein=FALSE;
 			break;
 
+		case 'r':
+#ifdef HAVE_DEV_RANDOM
+			real_random_flag=TRUE;
+			break;
+#else
+                        fprintf (stderr, _("This system does not support real random data.\n"));
+                        exit(1);
+#endif
+
 		case 'F':
 			force=TRUE;
 			break;
@@ -196,13 +214,27 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 
 		case 'q':
 			quiet=TRUE;
-			break;			
+			break;
+
+		case 'x':
+                        if (bit7_flag==BITS7) {
+                          fprintf (stderr, _("--7bit and --hex parameters are mutually exclusive.\n"));
+                          fprintf (stderr, _("Specify the --8bit parameter since 7bit is the default.\n"));
+                          exit(1);
+                         }
+                        bit7_flag=BITS8;
+			hexflag=TRUE;
+			break;
 
 		case 'V':
 			quiet=FALSE;
 			break;
 
 		case '7':
+                       if (hexflag==TRUE) {
+                         fprintf (stderr, _("--7bit and --hex parameters are mutually exclusive.\n"));
+                         exit(1);
+                        }
 			bit7_flag=BITS7;
 			break;			
 
@@ -212,7 +244,7 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 
 		case 'f':
                         if (kin==1) {
-                          fprintf(stderr, _("--key and --file parameters are mutually exclusive.\n"));
+                          fprintf(stderr, _("--key and --keyfile parameters are mutually exclusive.\n"));
                           exit(1);
                         }
 			kin=2;
@@ -222,12 +254,14 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
 
 		case 'k':
                         if (kin==2) {
-                          fprintf(stderr, _("--key and --file parameters are mutually exclusive.\n"));
+                          fprintf(stderr, _("--key and --keyfile parameters are mutually exclusive.\n"));
                           exit(1);
                         }
 			kin=1;
-			keyword= malloc(strlen(optarg)+10);
-			strcpy(keyword,optarg);
+                        keyfile=malloc(1*sizeof(char*));
+                        keyfile[0]=malloc(strlen(optarg)+10);
+                        keyword=keyfile[0];
+			strcpy(keyword,optarg);                        
 			break;			
 
 		case 'm':
@@ -329,6 +363,9 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
   umask(066);
 #endif
 
+/* 7 bits in the key */
+if (bit7_flag==-1) bit7_flag=BITS7; /* The default behaviour */
+
 /* Normal startup */
 
 
@@ -350,7 +387,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n\n"));
     if (stream_flag==TRUE) file_count++;
 
 
-
 /* Do as many times as many files we got */
 for(i=0;i<file_count;i++) {
 
@@ -604,7 +640,7 @@ return (return_val);
 
 /* UNIX CRYPT Encryption */
 
-int encrypt_ucrypt(char *fromfile,char *tofile,char *key)
+int encrypt_ucrypt(char *fromfile, char *tofile, char *key)
 {
 word32 plaintext[2]; /* 64 bit */
 int ch=0,how=8; /* how needs to be initialized to 8 */
@@ -638,7 +674,7 @@ else {
 	if (write_lock( fileno((FILE*)TOF))==-1) return 1;
 }
 
-        Bzero(keyword,sizeof(keyword));
+         Bzero(keyword,sizeof(keyword));
 
 	if (key==NULL) 	{
           if (quiet==TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
@@ -711,22 +747,21 @@ return 0;
 
 /* General Encryption */
 
-int encrypt_general(int algorithm, char *fromfile,char *tofile,char *key)
+int encrypt_general(int algorithm, char *fromfile, char *tofile, char *key)
 {
 unsigned char counter=0;
 word32 fcrc32;
+word32 *keyword;
 int i=0,j=0,ch=0,how=0;
-char *tmp,*copychar;
+char *copychar;
 unsigned int len=0;
 FILE *RTOF;
 FILE *FROMF;
 int keysize=get_key_size(algorithm);
 int blocksize=get_block_size(algorithm);
-char tmpkey[128];
 int td;
 
 word32 *IV=0;
-word32 *keyword=calloc(keysize,1); 
 word32 *ciphertext=malloc(blocksize);
 FILE * TOF; 
 
@@ -742,7 +777,12 @@ if (mode==CBC || mode==CFB) {
 /* Random number generator */
 
  for (i=0;i<blocksize/sizeof(word32);i++) {
-  IV[i] = RAND32 ;
+    if (real_random_flag==TRUE) {
+       IV[i] = SRAND32;
+    }
+    else {
+       IV[i] = RAND32;
+    }
  }
 }
 
@@ -770,40 +810,20 @@ if (mode==CBC || mode==CFB) {
      /* Just leave space to put later the crc32 */
   }  
 
+  if (key==NULL) {
+    if (quiet==TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
+  }
 
-	if (key==NULL) 	{
-          if (quiet==TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
-          tmp=get_password( keysize, bit7_flag, ENCRYPT, algorithm , &len);
-          if (tmp==NULL) return 1;
 
-          if (len > keysize) len=keysize;
-          memmove(keyword, tmp, len);
-          Bzero(tmp, len);
-                
-	}
-	else {
-
-           if (bit7_flag==BITS8) {
-               len=strlen(key);
-
-               if (len>keysize) {
-                  fprintf(stderr,_("Warning: Reducing the size of the key\n"));
-                  len=keysize;
-               }
-               memmove(keyword, key, len);
-           }
-           else { /* 7bit mode */
-               len=strlen(key);
-               memmove(tmpkey, key, len);
-               len=strip8bit(tmpkey, len);
-               if (len > keysize) {
-                  fprintf(stderr,_("Warning: Reducing the size of the key\n"));
-                  len=keysize;
-               }
-               memmove(keyword, tmpkey, len);
-               Bzero(tmpkey, len); /* key is not bzero'd because it may be used again */
-           }           
-	}
+/* Get keyword */
+  if (key==NULL) {
+    keyword=fixkey( NULL, &len, bit7_flag, hexflag, algorithm, quiet, stream_flag, ENCRYPT);
+  }
+  else {
+    len=strlen(key);
+    keyword=fixkey( key, &len, bit7_flag, hexflag, algorithm, quiet, stream_flag, ENCRYPT);
+  }
+ if (keyword==NULL) return 1;
 
  if (quiet==FALSE) show_mode(ENCRYPT,algorithm,mode,tofile,bit7_flag);
 
@@ -962,7 +982,12 @@ switch (mode) {
 /* encrypt the crc32, but first put some random data there 
  */
    for(j=0; j<blocksize/sizeof(word32); j++) {
-     ciphertext[j] = RAND32;
+     if (real_random_flag==TRUE) {
+       ciphertext[j] = SRAND32;
+     }
+     else {
+       ciphertext[j] = RAND32;
+     }     
    }
    memmove(ciphertext,&fcrc32,sizeof(fcrc32));
    mcrypt(td, ciphertext);
@@ -1031,22 +1056,20 @@ switch (mode) {
 
 /* General Decryption */
 
-int decrypt_general(int algorithm, char *fromfile,char *tofile,char *key)
+int decrypt_general(int algorithm, char *fromfile, char *tofile, char *key)
 {
 word32 fcrc32, newcrc32;
 unsigned char counter=0;
 int i=0,ch=0,start=0;
-char *tmp;
 int keysize=get_key_size(algorithm);
 int blocksize=get_block_size(algorithm);
-char tmpkey[128];
 int td;
 
 word32 *ciphertext_old=malloc(blocksize);
 word32 *buffer=malloc(blocksize);
 word32 *ciphertext=malloc(blocksize);
 word32 *ciphertext_orig=malloc(blocksize);
-word32 *keyword=calloc(1,keysize);
+word32 *keyword;
 
 unsigned int len=0;
 
@@ -1074,40 +1097,22 @@ else {
 }
 
 
-  
-	if (key==NULL) {
-          if (quiet==TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
-          tmp=get_password(keysize,bit7_flag,DECRYPT,algorithm, &len);
-          if (tmp==NULL) return 1;
-                              
-          if (len>keysize) len=keysize;
-          memmove(keyword, tmp, len);
-          Bzero(tmp, len);
+/* Get key */
+  if (key==NULL) {
+    if (quiet==TRUE && stream_flag==FALSE) fprintf(stderr, _("File: %s\n"),fromfile);
+  }
 
-	}
-	else {
-            if (bit7_flag==BITS8) {
-              len=strlen(key);
-              if (len>keysize) {
-                 fprintf(stderr,_("Warning: Reducing the size of the key\n"));
-                 len=keysize;
-              }
-              memmove(keyword, key, len);
-            }
-            else { /* 7bit mode */
-               len=strlen(key);
-               memmove(tmpkey, key, len);
-               len=strip8bit( tmpkey, len);
-               if (len > keysize) {
-                  fprintf(stderr,_("Warning: Reducing the size of the key\n"));
-                  len=keysize;
-               }
-               memmove(keyword, tmpkey, len);
-               Bzero(tmpkey, len);
-            }          
-        }
+  if (key==NULL) {
+    keyword=fixkey( NULL, &len, bit7_flag, hexflag, algorithm, quiet, stream_flag, DECRYPT);
+  }
+  else {
+    len=strlen(key);
+    keyword=fixkey( key, &len, bit7_flag, hexflag, algorithm, quiet, stream_flag, DECRYPT);
+  }
+  if (keyword==NULL) return 1;
 
-if (quiet==FALSE) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);
+
+  if (quiet==FALSE) show_mode(DECRYPT,algorithm,mode,tofile,bit7_flag);
 
 
  
